@@ -10,6 +10,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -91,18 +92,25 @@ pop_singles (GtkWidget *vbox, GSList *list, GtkWidget *pw, gboolean visible)
                g_signal_connect (G_OBJECT (w), "key_press_event", 
 		         G_CALLBACK (phone_key_press_event), NULL);
             }
-          if (strcmp(e->tag,"NAME")) /* the name field on a button */
+          if (strcasecmp(e->tag, "NAME")) /* the name field on a button */
             {
               l = gtk_label_new (e->name);
               gtk_misc_set_alignment(GTK_MISC(l),1.0,0.5);
             }
           else
             {
-              l = gtk_button_new_with_label(e->name);
-              g_signal_connect(G_OBJECT(l),"clicked",
+              GtkWidget *b, *h;
+              h = gtk_hbox_new(FALSE, gpe_get_boxspacing());
+              l = gtk_label_new (e->name);
+              gtk_box_pack_start(GTK_BOX(h), w, TRUE, TRUE, 0);
+              GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
+              b = gtk_button_new_with_label(_("Details"));
+              gtk_box_pack_start(GTK_BOX(h), b, FALSE, TRUE, 0);
+              g_object_set_data(G_OBJECT(b), "edit", w);
+              w = h;
+              g_signal_connect(G_OBJECT(b),"clicked",
                                G_CALLBACK(on_name_clicked),
                                gtk_widget_get_toplevel(pw));
-              GTK_WIDGET_SET_FLAGS(w,GTK_CAN_DEFAULT);
             }
           gtk_table_attach (GTK_TABLE (table),
                     l,
@@ -204,7 +212,7 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw, gboolean visib
         case ITEM_DATE:
         {
           GtkWidget *l = gtk_label_new (e->name);
-          GtkWidget *hbox, *cbsched, *datecombo, *cbnoyear;
+          GtkWidget *hbox, *datecombo, *cbnoyear;
           pop_singles (vbox, singles, pw, visible);
           singles = NULL;
           
@@ -227,7 +235,7 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw, gboolean visib
             gtk_widget_show_all(hbox);
           else
             gtk_widget_hide_all(hbox);
-          gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, gpe_get_boxspacing());
+          gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, gpe_get_boxspacing());
           add_tag (e->tag, datecombo, pw);
         }
         break;
@@ -341,7 +349,6 @@ create_edit (void)
       gtk_container_add (GTK_CONTAINER (edit), vbox);
     }
   
-  gtk_window_set_title (GTK_WINDOW (edit), _("Edit Contact"));
   gpe_set_window_icon (edit, "icon");
 
   notebook2 = gtk_notebook_new ();
@@ -510,11 +517,16 @@ update_categories_list (GtkWidget *ui, GSList *selected, GtkWidget *edit)
 }
 
 void
-edit_person (struct person *p)
+edit_person (struct person *p, gboolean isnew)
 {
   GtkWidget *catlabel, *w = edit_window ();
   gchar *str;
   
+  if (isnew)
+    gtk_window_set_title (GTK_WINDOW (w), _("New Contact"));
+  else
+    gtk_window_set_title (GTK_WINDOW (w), _("Edit Contact"));
+      
   if (p)
     {
       GSList *tags = gtk_object_get_data (GTK_OBJECT (w), "tag-widgets");
@@ -527,7 +539,7 @@ edit_person (struct person *p)
           guint pos = 0;
           if (tag)
             {
-               if (!strcasecmp(tag,"NAME"))
+               if (!strcasecmp(tag, "NAME"))
                  {
                    gtk_widget_grab_default(w);
                    gtk_widget_grab_focus(w);
@@ -590,7 +602,7 @@ update_edit (struct person *p, GtkWidget *w)
   GtkWidget *nameentry[5];
   int namenum = 0;
   int i;
-  gchar *n1 = NULL, *n2 = NULL, *n3 = NULL, *n4 = NULL;
+  gchar *n1 = NULL, *n2 = NULL, *n3 = NULL, *n4 = NULL, *n5 = NULL;
   if (p)
     {
       GSList *tags = gtk_object_get_data (GTK_OBJECT (w), "tag-widgets");
@@ -604,15 +616,17 @@ update_edit (struct person *p, GtkWidget *w)
           if (v && v->value)
             {
                /* collect data for name field update */
-              if (!strcmp(v->tag,"TITLE"))
+              if (!strcasecmp(v->tag,"TITLE"))
                 n1 = v->value;
-              if (!strcmp(v->tag,"GIVEN_NAME"))
+              if (!strcasecmp(v->tag,"GIVEN_NAME"))
                 n2 = v->value;
-              if (!strcmp(v->tag,"FAMILY_NAME"))
+              if (!strcasecmp(v->tag,"MIDDLE_NAME"))
                 n3 = v->value;
-              if (!strcmp(v->tag,"HONORIFIC_SUFFIX"))
+              if (!strcasecmp(v->tag,"FAMILY_NAME"))
                 n4 = v->value;
-              if (!strcmp(v->tag,"NAME"))
+              if (!strcasecmp(v->tag,"HONORIFIC_SUFFIX"))
+                n5 = v->value;
+              if (!strcasecmp(v->tag,"NAME"))
               {
                 nameentry[namenum] = w;
                 namenum++;
@@ -620,7 +634,7 @@ update_edit (struct person *p, GtkWidget *w)
 
               if (GTK_IS_EDITABLE (w))
                 {
-                  gtk_editable_delete_text(GTK_EDITABLE(w),0,-1);
+                  gtk_editable_delete_text(GTK_EDITABLE(w), 0, -1);
                   gtk_editable_insert_text (GTK_EDITABLE (w), v->value,
                                             strlen (v->value), &pos);
                 }
@@ -652,7 +666,7 @@ update_edit (struct person *p, GtkWidget *w)
                     gtk_image_set_from_stock(GTK_IMAGE(w), 
                       GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_BUTTON);
                 }
-              else  
+              else if (GTK_IS_TEXT_VIEW(w))
                 gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (w)), 
                           v->value, -1);
             }
@@ -662,31 +676,180 @@ update_edit (struct person *p, GtkWidget *w)
   if (namenum)
     {
       gchar *ts = g_strdup(n1);
-      if (n2) 
+      if (n2)
         {
-          n1 = g_strdup_printf("%s %s",ts,n2);
+          n1 = g_strdup_printf("%s %s", ts, g_strstrip(n2));
           g_free(ts);
           ts = n1;
         }
       if (n3) 
         {
-          n1 = g_strdup_printf("%s %s",ts,n3);
+          n1 = g_strdup_printf("%s %s", ts, g_strstrip(n3));
           g_free(ts);
           ts = n1;
         }
       if (n4) 
         {
-          n1 = g_strdup_printf("%s %s",ts,n4);
+          n1 = g_strdup_printf("%s %s", ts, g_strstrip(n4));
           g_free(ts);
           ts = n1;
         }
+      if (n5) 
+        {
+          n1 = g_strdup_printf("%s %s", ts, g_strstrip(n5));
+          g_free(ts);
+          ts = n1;
+        }
+
       for (i=0;i<namenum;i++)
-        gtk_entry_set_text(GTK_ENTRY(nameentry[i]),g_strstrip(ts));
+        gtk_entry_set_text(GTK_ENTRY(nameentry[i]), g_strstrip(ts));
       g_free(ts);
     }
 }
 
 
+void
+store_name_fields(struct person *p, const char *nametext)
+{
+char e[4][100];
+int num_elements, i;
+gboolean has_title = FALSE, has_suffix = FALSE;
+char *sp;
+char *title = NULL;
+char *first = NULL;
+char *middle = NULL;
+char *last = NULL;
+char *suffix = NULL;
+   
+  if (!nametext) return;
+  if (strlen(nametext) < 3) return;
+      
+    sp = g_strdup(nametext);
+
+    /* scan first element for title */
+    if (sscanf(sp, "%s", e[0]))
+      {
+        i = 0;
+        struct tag_value *v;
+        while (titles[i])
+          {
+            if (!strcasecmp(e[0], titles[i]))
+              {
+                has_title = TRUE;
+                title = sp;
+                sp[strlen(e[0])] = 0;
+                sp += strlen(e[0])+1;
+                break;
+              }
+            i++;
+          }
+        if (!has_title) /* find user defined title */
+          {
+            v = db_find_tag(p, "TITLE");
+            if (v && !strcasecmp(e[0], v->value))
+              {
+                has_title = TRUE;
+                title = sp;
+                sp[strlen(v->value)] = 0;
+                sp += strlen(v->value)+1;
+              }
+          }
+      }
+    
+    /* strip trailing spaces */
+    while (strlen(sp) && isblank(sp[0]))
+      sp++;
+    
+    /* scan last element for suffix */
+    if ((long)(suffix = strrchr(sp, ' ') + 1) > 1)
+      {
+        i = 0;
+        while (suffixes[i])
+          {
+            if (!strcmp(suffix, suffixes[i]))
+              {
+                char *t;
+                has_suffix = TRUE;
+                t = strstr(sp, suffixes[i]);
+                if (t)
+                  {
+                    t--;
+                    t[0] = 0;
+                  }
+                break;
+              }
+            i++;
+          }
+      }
+    if (!has_suffix) 
+      suffix = NULL;
+   
+    /* count remaining elements */
+    num_elements = sscanf(sp, 
+                          "%100s %100s %100s %100s", 
+                          e[0], e[1], e[2], e[3]);
+    
+    switch (num_elements)
+      {
+        case 0: 
+        break;
+        case 1: 
+          last = e[0];
+        break;
+        case 2: 
+          if (strstr(sp, ","))
+            {
+              last = sp;
+              first = strstr(sp, ",") + 1;
+              while (strlen(first) && isblank(first[0]))
+                first++;
+              strstr(sp, ",")[0] = 0;
+            }
+          else
+            {
+              first = sp;
+              last = strrchr(sp, ' ') + 1;
+              while (strlen(last) && isblank(last[0]))
+                last++;
+              strrchr(sp, ' ')[0] = 0;
+            }
+        break;
+        default:
+          if (strstr(sp, ","))
+            {
+              last = sp;
+              first = strstr(sp, ",") + 1;
+              while (strlen(first) && isblank(first[0]))
+                first++;
+              strstr(sp, ",")[0] = 0;
+              
+              middle = strrchr(first, ' ') + 1;
+              while (strlen(middle) && isblank(middle[0]))
+                middle++;
+              strrchr(first, ' ')[0] = 0;
+            }
+          else
+            {
+              first = sp;
+              last = strrchr(sp, ' ') + 1;
+              while (strlen(last) && isblank(last[0]))
+                last++;
+              strrchr(sp, ' ')[0] = 0;
+              middle = strrchr(first, ' ') + 1;
+              while (strlen(middle) && isblank(middle[0]))
+                middle++;
+              strrchr(first, ' ')[0] = 0;
+            }
+        break;
+      }
+    db_set_data(p, "GIVEN_NAME", g_strdup(first));
+    db_set_data(p, "MIDDLE_NAME", g_strdup(middle));
+    db_set_data(p, "FAMILY_NAME", g_strdup(last));
+    db_set_data(p, "TITLE", g_strdup(title));
+    db_set_data(p, "HONORIFIC_SUFFIX", g_strdup(suffix));
+}
+
+
 void
 on_edit_save_clicked (GtkButton * button, gpointer user_data)
 {
@@ -744,50 +907,7 @@ on_edit_save_clicked (GtkButton * button, gpointer user_data)
 
     /* handle name details */
     if (nametext)
-      {
-          char e1[101];
-          char e2[101];
-          char e3[101];
-          char e4[101];
-          int num_elements = sscanf(nametext, "%100s %100s %100s %100s", e1, e2, e3, e4);
-          if (num_elements == 2)
-            {
-              if (strstr(nametext, ","))
-                {
-                  if (strstr(e1,",")) 
-                    strstr(e1,",")[0] = 0;
-                  db_set_data(p, "GIVEN_NAME", g_strdup(e2));
-                  db_set_data(p, "FAMILY_NAME", g_strdup(e1));
-                }
-              else
-                {
-                  db_set_data(p, "GIVEN_NAME", g_strdup(e1));
-                  db_set_data(p, "FAMILY_NAME", g_strdup(e2));
-                }
-            }
-              
-          if (num_elements > 2)
-            {
-              db_set_data(p, "TITLE", g_strdup(e1));
-              if (strstr(nametext, ","))
-                {
-                  if (strstr(e2,",")) 
-                    strstr(e2,",")[0] = 0;
-                  db_set_data(p, "GIVEN_NAME", g_strdup(e3));
-                  db_set_data(p, "FAMILY_NAME", g_strdup(e2));
-                }
-              else
-                {
-                  db_set_data(p, "GIVEN_NAME", g_strdup(e2));
-                  db_set_data(p, "FAMILY_NAME", g_strdup(e3));
-                }
-            }
-          if (num_elements > 3)
-            {
-              db_set_data(p, "HONORIFIC_SUFFIX", g_strdup(e4));
-            }
-      }
-  
+      store_name_fields(p, nametext);
   
   if (commit_person (p))
     {
@@ -926,8 +1046,15 @@ void
 on_name_clicked (GtkButton *button, gpointer user_data)
 {
   GtkWindow *edit = user_data;
+  GtkEntry *e = g_object_get_data(G_OBJECT(button), "edit");
   struct person *p;
+  gchar *name;
 
   p = g_object_get_data (G_OBJECT (edit), "person");
+  
+  name = gtk_editable_get_chars(GTK_EDITABLE(e), 0, -1);
+  store_name_fields(p, name);
+  g_free(name);
+
   do_edit_name_detail(edit, p);
 }
