@@ -8,7 +8,7 @@
  */
 
 /*
- * $Id: main.c,v 1.83 2004/08/25 14:55:07 florian Exp $
+ * $Id: main.c,v 1.90 2004/09/16 16:57:47 florian Exp $
  *
  */
 
@@ -55,6 +55,8 @@ GtkWidget *bluetooth_menu_item;
 guint menu_uid;
 gboolean mode_landscape;
 gboolean mode_large_screen;
+gboolean scroll_delay = FALSE;
+gboolean do_wrap = FALSE;
 
 struct gpe_icon my_icons[] = {
   {"edit"},
@@ -72,7 +74,7 @@ menu_do_edit (void)
 {
   struct person *p;
   p = db_get_by_uid (menu_uid);
-  edit_person (p);
+  edit_person (p, FALSE);
 }
 
 static void
@@ -82,7 +84,7 @@ menu_do_delete (void)
 			"question", "!gtk-cancel", NULL, "!gtk-delete", NULL, NULL) == 1)
     {
       if (db_delete_by_uid (menu_uid))
-	update_display ();
+        update_display ();
     }
 }
 
@@ -145,8 +147,14 @@ static void
 new_contact (GtkWidget * widget, gpointer d)
 {
   struct person *p;
+  GtkWidget *new_button, *edit_button;
+	  
+  new_button = g_object_get_data(G_OBJECT(mainw), "new-button");
+  edit_button = g_object_get_data(G_OBJECT(mainw), "edit-button");
+  gtk_widget_set_sensitive (new_button, FALSE);
+  gtk_widget_set_sensitive (edit_button, FALSE);
   p = new_person ();
-  edit_person (p);
+  edit_person (p, TRUE);
 }
 
 static void
@@ -160,9 +168,15 @@ edit_contact (GtkWidget * widget, gpointer d)
     {
       guint uid;
       struct person *p;
+      GtkWidget *new_button, *edit_button;
+	  
+	  new_button = g_object_get_data(G_OBJECT(mainw), "new-button");
+	  edit_button = g_object_get_data(G_OBJECT(mainw), "edit-button");
+      gtk_widget_set_sensitive (new_button, FALSE);
+      gtk_widget_set_sensitive (edit_button, FALSE);
       gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 1, &uid, -1);
       p = db_get_by_uid (uid);
-      edit_person (p);
+      edit_person (p, FALSE);
     }
 }
 
@@ -277,6 +291,7 @@ show_details (struct person *p)
             if ((fieldname) 
                  && strcasecmp(id->tag,"NAME")
                  && strcasecmp(id->tag,"GIVEN_NAME")
+                 && strcasecmp(id->tag,"MIDDLE_NAME")
                  && strcasecmp(id->tag,"FAMILY_NAME")
                  && strcasecmp(id->tag,"HONORIFIC_SUFFIX")
                  && strcasecmp(id->tag,"TITLE")
@@ -396,10 +411,11 @@ selection_made (GtkTreeSelection *sel, GObject *o)
   guint id;
   struct person *p;
   GtkTreeModel *model;
-  GtkWidget *edit_button, *delete_button;
+  GtkWidget *edit_button, *delete_button, *new_button;
 
   edit_button = g_object_get_data (o, "edit-button");
   delete_button = g_object_get_data (o, "delete-button");
+  new_button = g_object_get_data (o, "new-button");
 
   if (gtk_tree_selection_get_selected (sel, &model, &iter))
     {
@@ -419,20 +435,29 @@ selection_made (GtkTreeSelection *sel, GObject *o)
       gtk_widget_set_sensitive (edit_button, FALSE);
       gtk_widget_set_sensitive (delete_button, FALSE);
     }
+	/* restore after edit/new */
+    gtk_widget_set_sensitive (new_button, TRUE);
 }
 
 static gboolean
 match_for_search (struct person *p, const gchar *text, struct gpe_pim_category *cat)
 {
   gchar *gn;
+  gchar *mn;
   gchar *fn;
   gchar *name;
+  gchar *company;
   
   if (p->given_name)
     gn = g_utf8_strdown(p->given_name, -1);
   else
     gn = g_strdup("");
   
+  if (p->middle_name)
+    mn = g_utf8_strdown(p->middle_name, -1);
+  else
+    mn = g_strdup("");
+  
   if (p->family_name)
     fn = g_utf8_strdown(p->family_name, -1);
   else
@@ -443,17 +468,29 @@ match_for_search (struct person *p, const gchar *text, struct gpe_pim_category *
   else
     name = g_strdup("");
   
-  if (!g_str_has_prefix(gn,text) 
-      && !g_str_has_prefix(fn,text) && !g_str_has_prefix(name,text))
+  if (p->company)
+    company = g_utf8_strdown(p->company, -1);
+  else
+    company = g_strdup("");
+  
+  if (!g_str_has_prefix(gn, text) 
+      && !g_str_has_prefix(fn, text) 
+      && !g_str_has_prefix(name, text)
+      && !g_str_has_prefix(mn, text)
+      && !g_str_has_prefix(company, text))
     {
       if (gn) g_free(gn);
+      if (mn) g_free(mn);
       if (fn) g_free(fn);
-      if (fn) g_free(name);
-        return FALSE;
+      if (name) g_free(name);
+      if (company) g_free(company);
+      return FALSE;
     }
   if (gn) g_free(gn);
+  if (mn) g_free(mn);
   if (fn) g_free(fn);
-  if (fn) g_free(name);
+  if (name) g_free(name);
+  if (company) g_free(company);
     
 /* alternate method if nothing found 
   if ...
@@ -519,6 +556,7 @@ do_search (GObject *obj, GtkWidget *entry)
   all_entries = g_slist_sort (all_entries, (GCompareFunc)sort_entries);
 
   gtk_list_store_clear (list_store);
+  show_details(NULL);
 
   for (iter = all_entries; iter; iter = iter->next)
     {
@@ -582,6 +620,42 @@ update_display (void)
   do_search (G_OBJECT (search_entry), search_entry);
 }
 
+static gboolean
+scroll_timeout(gpointer d)
+{
+  do_wrap = TRUE;
+  scroll_delay = FALSE;
+  return FALSE;
+}
+
+void
+list_view_cursor_changed (GtkTreeView *treeview, gpointer user_data)
+{
+  GtkWidget *lStatus = user_data;
+  int pos;
+  int count =
+      gtk_tree_model_iter_n_children(GTK_TREE_MODEL(list_store), NULL);
+  GtkTreePath *path;
+  
+  if (lStatus)
+    {
+      gtk_tree_view_get_cursor(treeview, &path, NULL);
+      if (path)
+        {
+          gchar *text;
+          gint *index = gtk_tree_path_get_indices(path);
+          if (index)
+            {
+              pos = index[0] + 1;
+              text = g_strdup_printf("%d / %d", pos, count);
+              gtk_label_set_text(GTK_LABEL(lStatus), text);
+              g_free(text);
+            }
+          gtk_tree_path_free(path);
+        }
+    }
+}
+
 static gboolean
 window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkTreeView *tree)
 {
@@ -604,6 +678,10 @@ window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkTreeView *tree)
               new_contact(widget, NULL);
               return TRUE;
             break;
+            case GDK_d:
+              delete_contact(widget, NULL);
+              return TRUE;
+            break;
           }
     }
   
@@ -611,22 +689,61 @@ window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkTreeView *tree)
     {
       gtk_tree_view_get_cursor(GTK_TREE_VIEW(tree),&path,NULL);
       if (path) 
+      {
+        GtkTreePath *p;
+        int count =
+          gtk_tree_model_iter_n_children(GTK_TREE_MODEL(list_store), NULL); /* count */
+        p = gtk_tree_path_copy(path);
         gtk_tree_path_prev(path);
+        if (!gtk_tree_path_compare(path, p))  /* same? wrap around */
+        {
+          if (do_wrap)
+            {
+              path = gtk_tree_path_new_from_indices(count - 1, -1);
+              do_wrap = FALSE;
+            }
+          else
+            if (!scroll_delay)
+              {
+                g_timeout_add(300, scroll_timeout, NULL);
+                scroll_delay = TRUE;
+              }
+        }
+        gtk_tree_path_free(p);  
+      }
       else
         path = gtk_tree_path_new_first();
-      gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree),path,NULL,FALSE);
+      
+      gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree), path, NULL, FALSE);
       gtk_tree_path_free(path);
+      do_wrap = FALSE;
       return TRUE;
     }
   if (k->keyval == GDK_Down)
     {
-      gtk_tree_view_get_cursor(GTK_TREE_VIEW(tree),&path,NULL);
+      gtk_tree_view_get_cursor(GTK_TREE_VIEW(tree), &path, NULL);
       if (path)
         gtk_tree_path_next(path);
-      else
-        path = gtk_tree_path_new_first();
-      gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree),path,NULL,FALSE);
-      gtk_tree_path_free(path);
+      else /* wrap around */
+        {    
+          if (do_wrap)
+            {
+              path = gtk_tree_path_new_first();
+              do_wrap = FALSE;
+            }
+          else
+            if (!scroll_delay)
+              {
+                g_timeout_add(400, scroll_timeout, NULL);
+                scroll_delay = TRUE;
+              }
+        }
+      if (path)
+        {
+          gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree), path, NULL, FALSE);
+          gtk_tree_path_free(path);
+        }
+      do_wrap = FALSE;
       return TRUE;
     }
 
@@ -668,7 +785,7 @@ search_entry_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkWidget *tool
 static gboolean
 list_button_press_event (GtkWidget *widget, GdkEventButton *b, GtkListStore *list_store)
 {
-  if (b->button == 3)
+  if ((b->button == 3) || ((b->button == 1) && (b->type == GDK_2BUTTON_PRESS)))
     {
       gint x, y;
       GtkTreePath *path;
@@ -680,25 +797,33 @@ list_button_press_event (GtkWidget *widget, GdkEventButton *b, GtkListStore *lis
 					 x, y,
 					 &path, NULL,
 					 NULL, NULL))
-	{
-	  GtkTreeIter iter;
-	  guint id;
-
-	  gtk_tree_model_get_iter (GTK_TREE_MODEL (list_store), &iter, path);
-
-	  gtk_tree_path_free (path);
-
-	  gtk_tree_model_get (GTK_TREE_MODEL (list_store), &iter, 1, &id, -1);
-
-	  menu_uid = id;
-
-	  if (export_bluetooth_available ())
-	    gtk_widget_show (bluetooth_menu_item);
-	  else
-	    gtk_widget_hide (bluetooth_menu_item);
-
-	  gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL, NULL, widget, b->button, b->time);
-	}
+        {
+          GtkTreeIter iter;
+          guint id;
+      
+          gtk_tree_model_get_iter (GTK_TREE_MODEL (list_store), &iter, path);
+      
+          gtk_tree_path_free (path);
+          
+          if (b->button == 3)
+            {
+              gtk_tree_model_get (GTK_TREE_MODEL (list_store), &iter, 1, &id, -1);
+          
+              menu_uid = id;
+          
+              if (export_bluetooth_available ())
+                gtk_widget_show (bluetooth_menu_item);
+              else
+                gtk_widget_hide (bluetooth_menu_item);
+          
+              gtk_menu_popup (GTK_MENU (popup_menu), 
+                              NULL, NULL, NULL, widget, b->button, b->time);
+            }
+          else
+            {
+                edit_contact(widget, NULL);
+            }
+        }
     }
 
   return FALSE;
@@ -763,6 +888,7 @@ create_main (gboolean show_config_button)
   GtkTreeViewColumn *column;
   GtkTreeSelection *tree_sel;
   GtkWidget *scrolled_window;
+  GtkWidget *lStatus = NULL;
   gint size_x, size_y;
 
   /* screen layout detection */
@@ -792,6 +918,7 @@ create_main (gboolean show_config_button)
   btnNew = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_NEW,
 			    _("New contact"), _("Tap here to add a new contact."),
 			    G_CALLBACK (new_contact), NULL, -1);
+  g_object_set_data (G_OBJECT (main_window), "new-button", btnNew);
 
   pw = gtk_image_new_from_pixbuf (gpe_find_icon_scaled ("edit", 
 							gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar))));
@@ -843,6 +970,14 @@ create_main (gboolean show_config_button)
   renderer = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (_("Contact"), renderer,
 						     "text", 0, NULL);
+  if (mode_landscape)
+    {
+      /* min width = 20% */
+      gtk_tree_view_column_set_min_width(column, size_x / 5);
+      /* max width = 50% */
+      gtk_tree_view_column_set_max_width(column, size_x / 2); 
+    }
+  gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_view), column);
 
   hbox3 = gtk_hbox_new (FALSE, 0);
@@ -853,11 +988,17 @@ create_main (gboolean show_config_button)
     }
   gtk_box_pack_start (GTK_BOX (vbox1), hbox3, FALSE, FALSE, 0);
 
+  if (mode_large_screen)
+    {
+      lStatus = gtk_label_new (NULL);
+      gtk_misc_set_alignment(GTK_MISC(lStatus), 1.0, 0.5);
+      gtk_box_pack_end (GTK_BOX (hbox3), lStatus, FALSE, FALSE, 3);
+    }
   label83 = gtk_label_new (_("Find:"));
   gtk_box_pack_start (GTK_BOX (hbox3), label83, FALSE, FALSE, 0);
 
   entry1 = gtk_entry_new ();
-  gtk_widget_set_size_request(entry1,60,-1);
+  gtk_widget_set_size_request(entry1, 60, -1);
   gtk_box_pack_start (GTK_BOX (hbox3), entry1, FALSE, FALSE, 0);
 
   label84 = gtk_label_new (_("in"));
@@ -876,6 +1017,10 @@ create_main (gboolean show_config_button)
 
   g_signal_connect (G_OBJECT (list_view), "button_release_event", 
 		    G_CALLBACK (list_button_release_event), list_store);
+  
+  if (mode_large_screen)
+    g_signal_connect (G_OBJECT (list_view), "cursor-changed", 
+		    G_CALLBACK (list_view_cursor_changed), lStatus);
 
   gtk_widget_set_events (main_window,GDK_KEY_PRESS_MASK);
   gtk_widget_set_events (entry1,GDK_KEY_PRESS_MASK);
@@ -915,6 +1060,7 @@ create_main (gboolean show_config_button)
 
   displaymigration_mark_window (main_window);
 
+  
   return main_window;
 }
 
@@ -944,6 +1090,7 @@ main (int argc, char *argv[])
   int arg;
   gboolean show_config_button = TRUE;
   gchar *ifile = NULL;
+  GtkTreePath *path;
   
 #ifdef ENABLE_NLS
   setlocale (LC_ALL, "");
@@ -1025,6 +1172,10 @@ main (int argc, char *argv[])
   gtk_widget_show_all (mainw);
   gtk_widget_grab_focus (GTK_WIDGET (g_object_get_data (G_OBJECT (mainw), "entry")));
 
+  path = gtk_tree_path_new_first();
+  gtk_tree_view_set_cursor(GTK_TREE_VIEW(list_view), path, NULL, FALSE);
+  gtk_tree_path_free(path);
+
   gtk_main ();
   return 0;
 }
